@@ -341,6 +341,8 @@ static unsigned int parse_fg_colors(const std::string &s)
     return Config::FG_COLORS_WEB;
   else if (s == "default")
     return Config::FG_COLORS_DEFAULT;
+  else if (s == "black")
+    return Config::FG_COLORS_BLACK;
   long n = string::as<long>(s);
   if (n < 1 || n > static_cast<long>(djvu::max_fg_colors))
     throw FgColorsOutsideRange(1, djvu::max_fg_colors);
@@ -425,7 +427,7 @@ void Config::read_config(int argc, char * const argv[])
   static struct option options [] =
   {
     { "anti-alias", 0, 0, OPT_ANTIALIAS },
-    { "antialias", 0, 0, OPT_ANTIALIAS },
+    { "antialias", 0, 0, OPT_ANTIALIAS }, /* deprecated alias */
     { "bg-slices", 1, 0, OPT_BG_SLICES },
     { "bg-subsample", 1, 0, OPT_BG_SUBSAMPLE },
     { "crop-text", 0, 0, OPT_TEXT_CROP },
@@ -435,10 +437,10 @@ void Config::read_config(int argc, char * const argv[])
     { "guess-dpi", 0, 0, OPT_GUESS_DPI },
     { "help", 0, 0, OPT_HELP },
     { "hyperlinks", 1, 0, OPT_HYPERLINKS },
-    { "indirect", 0, 0, OPT_INDIRECT },
+    { "indirect", 1, 0, OPT_INDIRECT },
     { "jobs", 1, 0, OPT_JOBS },
     { "lines", 0, 0, OPT_TEXT_LINES },
-    { "loss-level", 1, 0, OPT_LOSS_ANY },
+    { "loss-level", 1, 0, OPT_LOSS_ANY }, /* deprecated alias */
     { "losslevel", 1, 0, OPT_LOSS_ANY },
     { "lossy", 0, 0, OPT_LOSS_100 },
     { "media-box", 0, 0, OPT_MEDIA_BOX },
@@ -659,6 +661,7 @@ void Config::usage(const Config::Error &error) const
     << std::endl <<   "     --bg-subsample=N"
     << std::endl <<   "     --fg-colors=default"
     << std::endl <<   "     --fg-colors=web"
+    << std::endl <<   "     --fg-colors=black"
 #if HAVE_GRAPHICSMAGICK
     << std::endl <<   "     --fg-colors=N"
 #endif
