@@ -1,4 +1,4 @@
-/* Copyright © 2009 Jakub Wilk
+/* Copyright © 2009, 2010 Jakub Wilk
  *
  * This package is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -11,7 +11,7 @@
 #include "paths.hh"
 #include "system.hh"
 
-#ifdef ENABLE_NLS
+#if ENABLE_NLS
 
 void i18n::setup_locale()
 {
