@@ -27,7 +27,7 @@
 #include "debug.hh"
 #include "djvuconst.hh"
 #include "paths.hh"
-#include "quantizer.hh"
+#include "image-filter.hh"
 #include "sexpr.hh"
 #include "system.hh"
 #include "version.hh"
@@ -251,6 +251,7 @@ protected:
   std::auto_ptr<std::ostringstream> text_comments;
   std::vector<sexpr::Ref> annotations;
   const ComponentList &page_files;
+  bool skipped_elements;
 
   void add_text_comment(int ox, int oy, int dx, int dy, int x, int y, int w, int h, const Unicode *unistr, int len)
   {
@@ -287,6 +288,7 @@ public:
   void drawImageMask(pdf::gfx::State *state, pdf::Object *object, pdf::Stream *stream, int width, int height,
     pdf::Bool invert, pdf::Bool inline_image)
   {
+    this->skipped_elements = true;
     return;
   }
 
@@ -294,7 +296,10 @@ public:
     pdf::gfx::ImageColorMap *color_map, int *mask_colors, pdf::Bool inline_image)
   {
     if (is_foreground_color_map(color_map) || config.no_render)
+    {
+      this->skipped_elements = true;
       return;
+    }
     Renderer::drawImage(state, object, stream, width, height, color_map, mask_colors, inline_image);
   }
 
@@ -302,7 +307,10 @@ public:
     pdf::gfx::ImageColorMap *color_map, pdf::Stream *mask_stream, int mask_width, int mask_height, pdf::Bool mask_invert)
   {
     if (is_foreground_color_map(color_map) || config.no_render)
+    {
+      this->skipped_elements = true;
       return;
+    }
     Renderer::drawMaskedImage(state, object, stream, width, height,
       color_map, mask_stream, mask_width, mask_height, mask_invert);
   }
@@ -312,7 +320,10 @@ public:
     int mask_width, int mask_height, pdf::gfx::ImageColorMap *mask_color_map)
   {
     if (is_foreground_color_map(color_map) || config.no_render)
+    {
+      this->skipped_elements = true;
       return;
+    }
     Renderer::drawSoftMaskedImage(state, object, stream, width, height,
       color_map, mask_stream, mask_width, mask_height, mask_color_map);
   }
@@ -322,6 +333,7 @@ public:
   void drawImageMask(pdf::gfx::State *state, pdf::Object *object, pdf::Stream *stream, int width, int height,
     pdf::Bool invert, pdf::Bool interpolate, pdf::Bool inline_image)
   {
+    this->skipped_elements = true;
     return;
   }
 
@@ -329,7 +341,10 @@ public:
     pdf::gfx::ImageColorMap *color_map, pdf::Bool interpolate, int *mask_colors, pdf::Bool inline_image)
   {
     if (is_foreground_color_map(color_map) || config.no_render)
+    {
+      this->skipped_elements = true;
       return;
+    }
     Renderer::drawImage(state, object, stream, width, height, color_map, interpolate, mask_colors, inline_image);
   }
 
@@ -338,7 +353,10 @@ public:
     pdf::Stream *mask_stream, int mask_width, int mask_height, pdf::Bool mask_invert, pdf::Bool mask_interpolate)
   {
     if (is_foreground_color_map(color_map) || config.no_render)
+    {
+      this->skipped_elements = true;
       return;
+    }
     Renderer::drawMaskedImage(state, object, stream, width, height,
       color_map, interpolate,
       mask_stream, mask_width, mask_height, mask_invert, mask_interpolate);
@@ -350,7 +368,10 @@ public:
     pdf::gfx::ImageColorMap *mask_color_map, pdf::Bool mask_interpolate)
   {
     if (is_foreground_color_map(color_map) || config.no_render)
+    {
+      this->skipped_elements = true;
       return;
+    }
     Renderer::drawSoftMaskedImage(state, object, stream, width, height,
       color_map, interpolate,
       mask_stream, mask_width, mask_height, mask_color_map, mask_interpolate);
@@ -368,7 +389,11 @@ public:
     state->transform(x, y, &pox, &poy);
     state->transformDelta(dx, dy, &pdx, &pdy);
     int old_render = state->getRender();
+    /* Setting the following rendering mode disallows drawing text, but allows
+     * fonts to be set up properly nevertheless:
+     */
     state->setRender(0x103);
+    this->skipped_elements = true;
     this->Renderer::drawChar(state, x, y, dx, dy, origin_x, origin_y, code, n_bytes, unistr, length);
     state->setRender(old_render);
     pdf::splash::Font *font = this->getCurrentFont();
@@ -523,12 +548,17 @@ public:
   }
 
   void stroke(pdf::gfx::State *state)
-  { }
+  {
+    this->skipped_elements = true;
+  }
 
   void fill(pdf::gfx::State *state)
   {
     if (config.no_render)
+    {
+      this->skipped_elements = true;
       return;
+    }
     pdf::splash::Path path;
     this->convert_path(state, path);
     double area = pdf::get_path_area(path);
@@ -580,6 +610,18 @@ public:
     this->text_comments.reset(new std::ostringstream);
     *(this->text_comments) << std::setfill('0');
   }
+
+  void clear()
+  {
+    this->skipped_elements = 0;
+    this->clear_texts();
+    this->clear_annotations();
+  }
+
+  bool has_skipped_elements()
+  {
+    return this->skipped_elements;
+  }
 };
 
 class BookmarkError : public std::runtime_error
@@ -1152,17 +1194,6 @@ static int xmain(int argc, char * const argv[])
     exit(1);
   }
 
-#if _OPENMP
-  if (config.n_jobs >= 1)
-    omp_set_num_threads(config.n_jobs);
-#else
-  if (config.n_jobs != 1)
-  {
-    debug(1) << string_printf(_("Warning: %s"), _("Multi-threading is not supported.")) << std::endl;
-    config.n_jobs = 1;
-  }
-#endif
-
   if (config.output_stdout && is_stream_a_tty(std::cout))
     throw StdoutIsATerminal();
 
@@ -1194,12 +1225,33 @@ static int xmain(int argc, char * const argv[])
   std::auto_ptr<Quantizer> quantizer;
   if (config.monochrome)
     quantizer.reset(new DummyQuantizer(config));
-  else if (config.fg_colors == Config::FG_COLORS_DEFAULT)
-    quantizer.reset(new DefaultQuantizer(config));
-  else if (config.fg_colors == Config::FG_COLORS_WEB)
-    quantizer.reset(new WebSafeQuantizer(config));
   else
-    quantizer.reset(new GraphicsMagickQuantizer(config));
+    switch (config.fg_colors)
+    {
+    case Config::FG_COLORS_DEFAULT:
+      quantizer.reset(new DefaultQuantizer(config));
+      break;
+    case Config::FG_COLORS_WEB:
+      quantizer.reset(new WebSafeQuantizer(config));
+      break;
+    case Config::FG_COLORS_BLACK:
+      quantizer.reset(new MaskQuantizer(config));
+      break;
+    default:
+      quantizer.reset(new GraphicsMagickQuantizer(config));
+    }
+
+#if _OPENMP
+  if (config.n_jobs >= 1)
+    omp_set_num_threads(config.n_jobs);
+#else
+  if (config.n_jobs != 1)
+  {
+    debug(1) << string_printf(_("Warning: %s"), _("pdf2djvu was built without OpenMP support; multi-threading is disabled.")) << std::endl;
+    config.n_jobs = 1;
+  }
+#endif
+
   if (config.format == config.FORMAT_BUNDLED)
   {
     if (config.output_stdout)
@@ -1256,7 +1308,7 @@ static int xmain(int argc, char * const argv[])
   for (int n = page_range->first; n <= n_pages && n <= page_range->second; n++)
   {
     static int i = 1;
-    page_map.set(i, n);
+    page_map.set(n, i);
     page_numbers.push_back(n);
     i++;
   }
@@ -1320,8 +1372,8 @@ static int xmain(int argc, char * const argv[])
     int height = outm->getBitmapHeight();
     n_pixels += width * height;
     debug(2) << string_printf(_("image size: %dx%d"), width, height) << std::endl;
-    if (!config.no_render)
-    {
+    if (!config.no_render && outm->has_skipped_elements())
+    { /* Render the page second time, without skipping any elements. */
       debug(3) << _("rendering page (2nd pass)") << std::endl;
       doc->display_page(out1.get(), n, dpi, dpi, crop, false);
     }
@@ -1333,10 +1385,17 @@ static int xmain(int argc, char * const argv[])
     int background_color[3];
     bool has_foreground = false;
     bool has_text = false;
-    (*quantizer)(out1.get(), outm.get(), width, height, background_color, has_foreground, has_background, sep_file);
+    (*quantizer)(
+        outm->has_skipped_elements() ? out1.get() : outm.get(),
+        outm.get(),
+        width, height,
+        background_color, has_foreground, has_background,
+        sep_file
+    );
     bool nonwhite_background_color;
     if (has_background)
     {
+      /* The image has a real (non-solid) background. Store subsampled IW44 image. */
       int sub_width, sub_height;
       calculate_subsampled_size(width, height, config.bg_subsample, sub_width, sub_height);
       double hdpi = sub_width / page_width;
@@ -1357,6 +1416,7 @@ static int xmain(int argc, char * const argv[])
     }
     else
     {
+      /* Background is solid. */
       nonwhite_background_color = (background_color[0] & background_color[1] & background_color[2] & 0xff) != 0xff;
       if (nonwhite_background_color)
       { /* Create a dummy background, just to assure existence of FGbz chunks.
