@@ -1,4 +1,4 @@
-/* Copyright © 2007, 2008, 2009 Jakub Wilk
+/* Copyright © 2007, 2008, 2009, 2010 Jakub Wilk
  * Copyright © 2009 Mateusz Turcza
  *
  * This package is free software; you can redistribute it and/or modify
@@ -23,7 +23,7 @@
 #include <sys/stat.h>
 #include <sys/types.h>
 
-#ifdef WIN32
+#if WIN32
 #include <climits>
 #include <windows.h>
 #endif
@@ -32,12 +32,17 @@
 #include "debug.hh"
 #include "i18n.hh"
 
+#if !HAVE_PSTREAMS && HAVE_FORK
+#include <sys/wait.h>
+#include <unistd.h>
+#endif
+
 /* constants
  * =========
  */
 
 static const char unix_path_separator = '/';
-#ifndef WIN32
+#if !WIN32
 static const char path_separator = unix_path_separator;
 #else
 static const char path_separator = '\\';
@@ -51,7 +56,7 @@ static const char path_separator = '\\';
 std::string POSIXError::error_message(const std::string &context)
 {
   std::string message;
-#ifdef WIN32
+#if WIN32
   /* Win32 systems tends to return POSIX-style error messages in English.
    * Translation is required.
    */
@@ -132,7 +137,7 @@ static void warn_posix_error(const std::string &context)
 {
   try
   {
-    throw_posix_error(context); 
+    throw_posix_error(context);
   }
   catch (const POSIXError &e)
   {
@@ -140,7 +145,7 @@ static void warn_posix_error(const std::string &context)
   }
 }
 
-#ifdef WIN32
+#if WIN32
 
 /* class Win32Error : OSError
  * ==========================
@@ -151,7 +156,7 @@ class Win32Error : public OSError
 protected:
   static std::string error_message(const std::string &context);
 public:
-  explicit Win32Error(const std::string &context) 
+  explicit Win32Error(const std::string &context)
   : OSError(error_message(context))
   { };
 };
@@ -234,12 +239,12 @@ void Command::operator()(bool quiet)
   this->call(NULL, quiet);
 }
 
-#ifndef HAVE_PSTREAMS
+#if !HAVE_PSTREAMS
 static const std::string argv_to_command_line(const std::vector<std::string> &argv)
 /* Translate a sequence of arguments into a command line string. */
 {
   std::ostringstream buffer;
-#ifdef WIN32
+#if WIN32
   /* Using the same rules as the MS C runtime:
    *
    * 1) Arguments are delimited by white space, which is either a space or a
@@ -300,7 +305,7 @@ static const std::string argv_to_command_line(const std::vector<std::string> &ar
   for (std::vector<std::string>::const_iterator parg = argv.begin(); parg != argv.end(); parg++)
   {
     buffer << "'";
-    if (parg->find_first_of("\\\'") == std::string::npos) 
+    if (parg->find_first_of("\\\'") == std::string::npos)
       buffer << *parg;
     else
       for (std::string::const_iterator pch = parg->begin(); pch != parg->end(); pch++)
@@ -318,7 +323,7 @@ static const std::string argv_to_command_line(const std::vector<std::string> &ar
 #endif
 
 
-#if defined(HAVE_PSTREAMS)
+#if HAVE_PSTREAMS
 
 void Command::call(std::ostream *my_stdout, bool quiet)
 {
@@ -341,16 +346,17 @@ void Command::call(std::ostream *my_stdout, bool quiet)
   if (status != 0)
   {
     std::ostringstream message;
-    message << "system(\"";
-    message << this->command;
-    message << " ...\") failed";
+    message
+      << "system(\""
+      << this->command
+      << " ...\") failed";
     if (WIFEXITED(status))
       message << " with exit code " << WEXITSTATUS(status);
     throw CommandFailed(message.str());
   }
 }
 
-#elif defined(WIN32)
+#elif WIN32
 
 void Command::call(std::ostream *my_stdout, bool quiet)
 {
@@ -447,13 +453,15 @@ void Command::call(std::ostream *my_stdout, bool quiet)
       if (my_stdout != NULL)
         my_stdout->write(buffer, nbytes);
     }
+    CloseHandle(read_end); /* ignore errors */
   }
   if (status < 0)
   {
     std::ostringstream message;
-    message << "system(\"";
-    message << this->command;
-    message << " ...\") failed";
+    message
+      << "system(\""
+      << this->command
+      << " ...\") failed";
     throw Win32Error(message.str());
   }
 }
@@ -491,15 +499,228 @@ void Command::call(std::ostream *my_stdout, bool quiet)
   if (status < 0)
   {
     std::ostringstream message;
-    message << "system(\"";
-    message << this->command;
-    message << " ...\") failed";
-    throw POSIXError(message.str());
+    message
+      << "system(\""
+      << this->command
+      << " ...\") failed";
+    throw_posix_error(message.str());
   }
 }
 
 #endif
 
+#if WIN32
+
+class FilterWriterData
+{
+public:
+  HANDLE handle;
+  const std::string &string;
+  FilterWriterData(HANDLE handle, const std::string &string)
+  : handle(handle),
+    string(string)
+  { }
+};
+
+unsigned long WINAPI filter_writer(void *data_)
+{
+  bool success;
+  FilterWriterData *data = reinterpret_cast<FilterWriterData*>(data_);
+  success = WriteFile(data->handle, data->string.c_str(), data->string.length(), NULL, NULL);
+  if (!success)
+    throw_win32_error("WriteFile");
+  success = CloseHandle(data->handle);
+  if (!success)
+    throw_win32_error("CloseHandle");
+  return 0;
+}
+
+std::string Command::filter(const std::string &command_line, const std::string string)
+{
+  int status = 0;
+  unsigned long rc;
+  HANDLE stdin_read, stdin_write, stdout_read, stdout_write, error_handle;
+  SECURITY_ATTRIBUTES security_attributes;
+  security_attributes.nLength = sizeof (SECURITY_ATTRIBUTES);
+  security_attributes.lpSecurityDescriptor = NULL;
+  security_attributes.bInheritHandle = true;
+  if (CreatePipe(&stdin_read, &stdin_write, &security_attributes, 0) == 0)
+    throw_win32_error("CreatePipe");
+  if (CreatePipe(&stdout_read, &stdout_write, &security_attributes, 0) == 0)
+    throw_win32_error("CreatePipe");
+  rc =
+    SetHandleInformation(stdout_read, HANDLE_FLAG_INHERIT, 0) &&
+    SetHandleInformation(stdin_write, HANDLE_FLAG_INHERIT, 0);
+  if (rc == 0)
+  {
+    if (GetLastError() == ERROR_CALL_NOT_IMPLEMENTED)
+    {
+      /* Presumably it's Windows 9x, so the call is not supported.
+       * Punt on security and let the pipe end be inherited.
+       */
+    }
+    else
+      throw_win32_error("SetHandleInformation");
+  }
+  error_handle = GetStdHandle(STD_ERROR_HANDLE);;
+  if (error_handle != INVALID_HANDLE_VALUE)
+  {
+    rc = DuplicateHandle(
+      GetCurrentProcess(), error_handle,
+      GetCurrentProcess(), &error_handle,
+      0, true, DUPLICATE_SAME_ACCESS
+    );
+    if (rc == 0)
+      throw_win32_error("DuplicateHandle");
+  }
+  {
+    PROCESS_INFORMATION process_info;
+    memset(&process_info, 0, sizeof process_info);
+    STARTUPINFO startup_info;
+    memset(&startup_info, 0, sizeof startup_info);
+    startup_info.cb = sizeof startup_info;
+    startup_info.hStdInput = stdin_read;
+    startup_info.hStdOutput = stdout_write;
+    startup_info.hStdError = error_handle;
+    startup_info.dwFlags = STARTF_USESTDHANDLES;
+    char *c_command_line = strdup(command_line.c_str());
+    if (c_command_line == NULL)
+      throw_posix_error("strdup");
+    rc = CreateProcess(
+      NULL, c_command_line,
+      NULL, NULL,
+      true, 0,
+      NULL, NULL,
+      &startup_info,
+      &process_info
+    );
+    free(c_command_line);
+    if (rc == 0)
+      status = -1;
+    else
+    {
+      CloseHandle(process_info.hProcess); /* ignore errors */
+      CloseHandle(process_info.hThread); /* ignore errors */
+      if (error_handle != INVALID_HANDLE_VALUE)
+        CloseHandle(error_handle); /* ignore errors */
+    }
+  }
+  if (status == 0)
+  {
+    std::ostringstream stream;
+    CloseHandle(stdin_read); /* ignore errors */
+    CloseHandle(stdout_write); /* ignore errors */
+    FilterWriterData writer_data(stdin_write, string);
+    HANDLE thread_handle = CreateThread(NULL, 0, filter_writer, &writer_data, 0, NULL);
+    if (thread_handle == NULL)
+      throw_win32_error("CreateThread");
+    while (true)
+    {
+      char buffer[BUFSIZ];
+      unsigned long nbytes;
+      bool success = ReadFile(stdout_read, buffer, sizeof buffer, &nbytes, NULL);
+      if (!success)
+      {
+        status = -(GetLastError() != ERROR_BROKEN_PIPE);
+        break;
+      }
+      stream.write(buffer, nbytes);
+    }
+    CloseHandle(stdout_read); /* ignore errors */
+    rc = WaitForSingleObject(thread_handle, INFINITE);
+    if (rc == WAIT_FAILED)
+      throw_win32_error("WaitForSingleObject");
+    CloseHandle(thread_handle); /* ignore errors */
+    return stream.str();
+  }
+  if (status < 0)
+  {
+    std::ostringstream message;
+    message
+      << "system(\""
+      << command_line
+      << "\") failed";
+    throw Win32Error(message.str());
+  }
+  return string; /* Should not really happen. */
+}
+
+#elif HAVE_FORK
+
+std::string Command::filter(const std::string &command_line, const std::string string)
+{
+  int rc;
+  int pipe_fds[2];
+  rc = pipe(pipe_fds);
+  if (rc != 0)
+    throw_posix_error("pipe");
+  pid_t writer_pid = fork();
+  if (writer_pid == -1)
+    throw_posix_error("fork");
+  else if (writer_pid == 0)
+  {
+    /* Writer: */
+    close(pipe_fds[0]); /* Deliberately ignore errors. */
+    rc = dup2(pipe_fds[1], STDOUT_FILENO);
+    if (rc == -1)
+      throw_posix_error("dup2");
+    FILE *fp = popen(command_line.c_str(), "w");
+    if (fp == NULL)
+      throw_posix_error("popen");
+    if (fputs(string.c_str(), fp) == EOF)
+      throw_posix_error("fputs");
+    rc = pclose(fp);
+    if (rc != 0)
+      throw_posix_error("pclose");
+    exit(0);
+  }
+  else
+  {
+    /* Main process: */
+    close(pipe_fds[1]); /* Deliberately ignore errors. */
+    std::ostringstream stream;
+    while (1)
+    {
+      char buffer[BUFSIZ];
+      size_t n = read(pipe_fds[0], buffer, sizeof buffer);
+      if (n < 0)
+        throw_posix_error("read");
+      else if (n == 0)
+        break;
+      else
+        stream.write(buffer, n);
+    }
+    int status;
+    writer_pid = wait(&status);
+    if (writer_pid == static_cast<pid_t>(-1))
+      throw_posix_error("wait");
+    if (status != 0)
+    {
+      std::ostringstream message;
+      message
+        << "system(\""
+        << command_line
+        << "\") failed";
+      if (WIFEXITED(status))
+        message << " with exit code " << WEXITSTATUS(status);
+      throw CommandFailed(message.str());
+    }
+    return stream.str();
+  }
+  return string; /* Should not really happen. */
+}
+
+#else
+
+std::string Command::filter(const std::string &command_line, const std::string string)
+{
+  /* Should not really happen. */
+  errno = ENOSYS;
+  throw_posix_error("fork");
+}
+
+#endif
+
 
 /* class Directory
  * ===============
@@ -507,7 +728,7 @@ void Command::call(std::ostream *my_stdout, bool quiet)
 
 Directory::Directory(const std::string &name)
 : name(name), posix_dir(NULL)
-{ 
+{
   this->open(name.c_str());
 }
 
@@ -591,7 +812,7 @@ public:
 
 TemporaryDirectory::TemporaryDirectory() : Directory()
 {
-#ifndef WIN32
+#if !WIN32
   TemporaryPathTemplate path_buffer;
   if (mkdtemp(path_buffer) == NULL)
     throw_posix_error(static_cast<char*>(path_buffer));
@@ -690,7 +911,7 @@ std::ostream &operator<<(std::ostream &out, const File &file)
 
 void TemporaryFile::construct()
 {
-#ifndef WIN32
+#if !WIN32
   TemporaryPathTemplate path_buffer;
   int fd = mkstemp(path_buffer);
   if (fd == -1)
@@ -730,7 +951,7 @@ TemporaryFile::~TemporaryFile() throw ()
  * =========================
  */
 
-ExistingFile::ExistingFile(const std::string &name) 
+ExistingFile::ExistingFile(const std::string &name)
 : File()
 {
   this->name = name;
@@ -746,7 +967,7 @@ ExistingFile::ExistingFile(const Directory& directory, const std::string &name)
   this->open(NULL, false);
 }
 
-#ifdef WIN32
+#if WIN32
 
 /* class Cwd
  * =========
@@ -798,7 +1019,7 @@ Cwd::~Cwd()
 
 namespace encoding
 {
-#ifdef WIN32
+#if WIN32
   /* The native encoding (so called ANSI character set) can differ from the
    * terminal encoding (typically: so called OEM charset).
    */
@@ -858,7 +1079,7 @@ namespace encoding
         }
         else
         {
-          /* An unpaired surrogate is encountered. */ 
+          /* An unpaired surrogate is encountered. */
           errno = EILSEQ;
           throw_posix_error(__FUNCTION__);
         }
@@ -1007,7 +1228,7 @@ std::string string_vprintf(const char *message, va_list args)
 
 void prevent_pop_out(void)
 {
-#ifdef WIN32
+#if WIN32
   /* GetConsoleProcessList() function is not available for some systems (e.g.,
    * Wine, Windows 98), so it's not desireable to import it at link time.
    */
